#--
# Copyright (c) 2010 Moodstocks SAS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# or in the LICENSE file which you should have received as part of
# this distribution.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#++

module MNotes class Note
  
  class << self
    
    def retrieve(id, user=nil)
      new(id, user)
    end
    
  end # class << self
  
  attr_reader :id, :user_id, :object_id, :text, :epoch, :count, :location, :vote
  
  def delete(u=nil)
    u ||= user
    u.jbody(:delete, "note/#{@id}")
    true
  end
  
  def initialize(h, user=nil)
    refresh(h, user)
  end
  
  def object
    MNotes::Object::retrieve(@object_id)
  end
  
  def refresh(h=nil, user=nil)
    h ||= @id
    if h.is_a?(String)
      user ||= User::last
      refresh( Map[user.jbody(:get, "note/#{h}")] )
    else
      raise "Object#refresh called with a #{h.class}" unless h.is_a?(Hash)
      h = Map(h)
      @id, @user_id, @object_id, @text, @epoch = h.values_at(:id, :user, :object, :text, :epoch)
      @count, @location, @vote = h.values_at(:count, :location, :vote)
      self
    end
  end
  
  def save(u=nil)
    u ||= user
    u.update_note(@id, @text, @location)
  end
  
  def to_hash
    h = {
      :id => @id,
      :user => @user_id,
      :object => @object_id,
      :text => @text,
      :epoch => @epoch,
      :count => @count,
    }
    h.merge!(:location => @location) if @location
    h.merge!(:vote => @vote) if @vote
  end
  
  def uid
    user_id
  end
  
  def user
    User::retrieve(@user_id)
  end
  
end end
